#if !defined(_AUTO_INDEX_H_)
#define _AUTO_INDEX_H_

/*!
 * \file
 * \brief Creació d'índexs automàtics
 * \author Toni Corvera
 *
 * $Id: AutoIndex.h 483 2013-06-17 17:41:04Z toni $
 *
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "CosResposta.h"
#include "portabilitat.h"
#include "Recurs.h"
#include "Uri.h"

#include <cassert>
#include <stdexcept>
#include <string>
#include <sstream>
#include <boost/asio.hpp>
#include <boost/filesystem.hpp>

namespace tfc {

/*!
 * \brief Generació del llistat d'un directori
 * \invariant \c directori() és un directori
 * \todo FIXME: \em race \em condition: directori desapareix
 */
class AutoIndex : public RecursDinamic {
public:
    /*!
     * \param u Url utilitzada per accedir a aquest índex
     * \param d Directori
     * \pre d és un directori
     */
    AutoIndex(const PeticioHTTP & p,
              const boost::asio::ip::tcp::endpoint & host_servidor,
              const Uri & u,
              const Recurs & d) // TODO: Normalitzar interficie
        : RecursDinamic(p),
          url_(u), dir(d), host_(host_servidor), cache_() // FIXME: dir -> dir_
    {
        using namespace boost::filesystem;
        assert( is_directory(d.fisica()) );
    }

    //! Constructor de còpia
    AutoIndex(const AutoIndex & a)
        : RecursDinamic(a), url_(a.url_), dir(a.dir), host_(a.host_), cache_(a.cache_)
    {
        // buit
    }

    const Recurs& recurs() const { return dir; }
    const Uri& url() const { return url_; }
    const boost::filesystem::path& directori() const { return dir.fisica(); }

    std::string html() const;

    void envia(boost::asio::ip::tcp::socket &) const throw (std::runtime_error);

    virtual tipus_capsaleres capsaleres() const;
protected:
    using CosResposta::envia;
private:
    //! \brief Operador d'assignació, no permesa
    AutoIndex& operator=(const AutoIndex & a) {
        throw ErrorNoImplementat(FITXER_I_LINIA_);
    }

    const Uri url_;
    const Recurs dir;
    const boost::asio::ip::tcp::endpoint host_;
    mutable std::string cache_;
};

} // ns tfc

#endif // _AUTO_INDEX_H_

// vim:set ts=4 et ai: //
