#if !defined(_TFC_BASE64_H_)
#define _TFC_BASE64_H_

/*!
 * \file
 * \author Toni Corvera
 *
 * $Id: Base64.h 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include <string>

namespace tfc {

/*!
 * \brief Codificació i descodificació de cadenes de text amb la codificació Base 64
 *
 * L'RFC 2068 (HTTP/1.1) dirigeix envers l'RFC 2045 (MIME) per a l'explicació
 * de la codificació.
 *
 * En la codificació es prenen els bytes d'entrada de 3 en 3 (24 bits), i s'interpreten
 * com a 4 valors de 6 bits (64 possibles valors). Cada un d'aquests 64 valors tè
 * assignat un caràcter (vegeu la taula \ref TRADUCCIO).
 * Si l'entrada no té una longitud divisible per 3 s'afegeixen un o dos bytes amb el
 * valor '=' a la cadena un cop codificada.
 *
 * \sa Base64 Content-Transfer-Encoding: http://tools.ietf.org/html/rfc2045#section-6.8
 */
namespace Base64 {

/*!
 * \brief Obté la versió codificada d'una cadena
 * \remarks La cadena generada contindrà els caràcters de \em padding adients
 * \post \code descodifica(codifica(x)) == x \endcode
 */
std::string codifica(const std::string & s);

/*!
 * \brief Obté la versió desodificada d'una cadena
 * \remarks Els caràcters fora de rang s'ignoraran
 * \remarks No cal que \p s tingui els caràcters de \em padding correctes
 * \post \code codifica(descodifica(x)) == x \endcode
 */
std::string descodifica(const std::string & s);

} // ns Base64

} // ns tfc

#endif // _TFC_BASE64_H_

// vim:set ts=4 et ai: //
