#if !defined(_TFCWEB_CGI_H_)
#define _TFCWEB_CGI_H_

/*!
 * \file
 * \author Toni Corvera
 *
 * $Id: CGI.h 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "CosResposta.h"
#include "Excepcions.h"
#include "PeticioHTTP.h"

#include <string>
#include <stdexcept>
#include <boost/filesystem.hpp>

namespace tfc {

// Rellevants:
// - Setting up CGI in NCSA httpd:
//   http://www.meb.uni-bonn.de/docs/tutorials/cgi.html
// - http://www.w3.org/CGI/

/*!
 * \brief Interfície cap a un script CGI
 */
class ProgramaCGI : public RecursDinamic {
public:
    /*!
     * \pre \p peticio.ruta().ruta() es correspon amb un fitxer llegible i executable
     */
    explicit ProgramaCGI(const PeticioHTTP & peticio,
                         const boost::filesystem::path & arrel_servidor)
                                                    throw (ErrorRutaInexistent);
    ProgramaCGI(const ProgramaCGI & c)
        : RecursDinamic(c),
          arrel_servidor_(c.arrel_servidor_), ruta_(c.ruta_)
    {
        // buit
    }
    ProgramaCGI& operator=(const ProgramaCGI & c) {
        RecursDinamic::operator=(c);
        arrel_servidor_ = c.arrel_servidor_;
        ruta_ = c.ruta_;
        return *this;
    }

    virtual std::string html() const { throw ErrorNoImplementat(FITXER_I_LINIA_); }

    virtual void envia(boost::asio::ip::tcp::socket & socket) const throw (std::runtime_error);

    virtual bool pot_generar_capsaleres() const { return true; }
protected:
    using CosResposta::envia;
private:
    boost::filesystem::path arrel_servidor_, ruta_;
    enum {
        MIDA_BUFFER = 8<<10
    };
};

} // ns tfc

#endif // _TFCWEB_CGI_H_

// vim:set ts=4 et ai: //
