
/*!
 * \file
 * \author Toni Corvera
 *
 * $Id: CapsaleraHTTP.cc 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "CapsaleraHTTP.h"

#include <algorithm>
#include <vector>
#include <boost/algorithm/string.hpp>
#include <boost/assign.hpp>

using namespace std;
using boost::algorithm::to_lower_copy;

namespace {

// Capçaleres que en l'RFC es defineixen amb múltiples valors (vegeu el bloc
// de comentaris al final d'aquest fitxer)
const vector<string> MULTI_VALOR = boost::assign::list_of
        ( "accept" )
        ( "accept-charset" )
        ( "accept-encoding" )
        ( "accept-language" )
        ( "accept-ranges" )
        ( "allow" )
        ( "cache-control" )
        ( "connection" )
        ( "content-encoding" )
        ( "content-language" )
        ( "expect" )
        ( "if-match" )
        ( "if-none-match" )
        ( "proxy-authenticate" )
        ( "range" )
        ( "te" )
        ( "trailer" )
        ( "transfer-encoding" )
        ( "upgrade" )
        ( "vary" )
        ;

} // ns anònim

namespace tfc {

void CapsaleraHTTP::set_valor(const string & nou_valor) {
    valor_ = nou_valor;
}

// static
bool CapsaleraHTTP::es_repetible(const string & nom) {
    return MULTI_VALOR.end() != std::find(MULTI_VALOR.begin(),
                                          MULTI_VALOR.end(),
                                          to_lower_copy(nom));
}

/* ----------------------------------------------------------- *
   Classificació de capçaleres, RFC 2068
   -----------------------------------------------------------

 - Generals (peticions i respostes), aplicables a missatge però no a cos
   pàg. 34
      Cache-Control
      Connection
      Date
      Pragma
      Transfer-Encoding
      Upgrade
      Via
 - De petició, aplicables a petició i al propi client
   pàg. 37
      Accept
      Accept-Charset
      Accept-Encoding
      Accept-Language
      Authorization
      From
      Host
      If-Modified-Since
      If-Match
      If-None-Match
      If-Range
      If-Unmodified-Since
      Max-Forwards
      Proxy-Authorization
      Range
      Referer
      User-Agent
  - De resposta, aplicables a futurs accessos al recurs, i al propi servidor
    pàg. 40
      Age                       [cache]
      Location
      Proxy-Authenticate
      Public
      Retry-After
      Server
      Vary
      Warning
      WWW-Authenticate
  - D'entitat, aplicables a l'entitat transferida (o sol·licitada)
    pàg. 41
      Allow
      Content-Base
      Content-Encoding
      Content-Language
      Content-Length
      Content-Location
      Content-MD5
      Content-Range
      Content-Type
      ETag
      Expires
      Last-Modified

* ------------------------------------------------------------ */

/* ----------------------------------------------------------- *
   Multiplicitat de valors,
    RFC 2068 a partir de pàg. 94
    RFC 2616 a partir de pàg. 100
    Els dos RFCs defineixen la multiplicitat d'algunes capçaleres de
    forma diferent, però només en capçaleres que accepten valors
    múltiples, de cara a aquest projecte això no és un problema
   -----------------------------------------------------------

   - Múltiples valors (separats per comes): #( valors ) = 0 a infinit
      Accept
      Allow
      TE

   - Múltiples valors (separats per comes): 1#( valors) = 1 a infinit
      Accept-Charset
      Accept-Encoding
      Accept-Language
      Accept-Ranges
      Cache-Control
      Connection
      Content-Encoding
      Content-Language
      Expect
      If-Match
      If-None-Match
      Proxy-Authenticate
      Range // Requereix un anàlisi especial per què el format no és simètric respecte la coma: "bytes=0-0, 10-20"
      Trailer
      Transfer-Encoding
      Upgrade
      Vary

   - Valors únics
      Age
      Authorization
      Content-Base // Eliminat en RFC 2616
      Content-Length
      Content-Location
      Content-MD5
      Content-Range
      Content-Type
      Date
      ETag
      Expires
      From
      Host
      If-Modified-Since
      If-Range
      If-Unmodified-Since
      Location
      Max-Forwards
      Proxy-Authorization
      Referer
      Retry-After
      Server
      User-Agent
      Via
      Warning
      WWW-Authenticate

* ------------------------------------------------------------ */

} // ns tfc

// vim:set ts=4 et ai: //
