#if !defined(_CAPSALERA_HTTP_H_)
#define _CAPSALERA_HTTP_H_

/*!
 * \file
 * \author Toni Corvera
 *
 * $Id: CapsaleraHTTP.h 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "portabilitat.h"

#include <iostream>
#include <ostream>
#include <stdexcept>
#include <string>

namespace tfc {

/*!
 * \brief Representació d'una capçalera HTTP
 * \deprecated
 */
class CapsaleraHTTP {
public:
    CapsaleraHTTP(const std::string & nom, const std::string & valor = "")
        : nom_(nom), valor_(valor)
    {
        // buit
    }
    /*!
     * \brief Constructor de còpia
     */
    CapsaleraHTTP(const CapsaleraHTTP & c)
        : nom_(c.nom()), valor_(c.valor())
    {
        // buit
    }

#if 0
    // Amb nom definit constant no es pot definir operator= coherentment
    // p.e. { CapsaleraHTTP("Host") h; h = CapsaleraHTTP("Server") } seria un problema
    CapsaleraHTTP& operator=(const CapsaleraHTTP & c) {
        if (this == &c) {
            return *this;
        }
        valor_ = c.valor();
        return *this;
    }
#endif

    bool operator==(const CapsaleraHTTP & c) const {
        if (this == &c) {
            return true;
        }
        return nom_ == c.nom() && valor_ == c.valor();
    }
    std::string nom() const { return nom_; }
    std::string valor() const { return valor_; }
    void set_valor(const std::string &);

    /*!
     * \brief Comprova si la capçalera \p nom pot aparèixer més d'una
     *        vegada (o una única vegada amb múltiples valors)
     * \remarks Les capçaleres desconegudes es consideren amb valor únic.
     */
    static bool es_repetible(const std::string & nom);
private:
    /* const */ std::string nom_;
    std::string valor_;

    friend std::ostream& operator<< (std::ostream& os, const CapsaleraHTTP& c) {
        return os << c.nom() << ": " << c.valor();
    }
};

} // ns tfc

#endif // _CAPSALERA_HTTP_H_

// vim:set ts=4 et ai: //
