
/*!
 * \file
 * \author
 *
 * $Id: ESQUELETO.cpp 198 2012-04-19 19:20:56Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "CodisEstatHTTP.h"

#include <cassert>
#include <boost/assign.hpp> // boost::assign::map_list_of
#include <boost/unordered_map.hpp>

using namespace boost;
using namespace std;

namespace {

using namespace tfc;

using boost::unordered_map;
using boost::hash;

// Hashing: unordered_map requereix una funció hash, i (almenys g++/clang)
//          no crea una per enum's
//          Solucions:
//           - Aprofitar que els enums són int's i:
//               - Definir el map com a unordered_map<int, string>
//               o
//               - Reutilitzar el hasher d'int
//           o
//           - Derivar tr1::hash per al tipus CodiEstatHTTP
//             Cal fer-ho en std::tr1 (en aquest cas és permés)

// FIXME: És legal enviar capçaleres UTF-8?
const unordered_map<CodiEstatHTTP, string, hash<int> > FRASES = assign::map_list_of
            ( CODI_100, "Continua" )
            ( CODI_101, "Canviant protocols" )
            ( CODI_200, "OK" )
            ( CODI_201, "Creat" )
            ( CODI_202, "Acceptat" )
            ( CODI_203, "Informació no autoritària" )
            ( CODI_204, "Sense contingut" )
            ( CODI_205, "Re-inici de contingut" )
            ( CODI_206, "Contingut parcial" )
            ( CODI_300, "Eleccions múltiples" )
            ( CODI_301, "Mogut permanentment" )
            ( CODI_302, "Mogut temporalment" )
            ( CODI_303, "Vegeu altres" )
            ( CODI_304, "No modificat" )
            ( CODI_305, "Utilitza proxy" )
            ( CODI_306, "" ) // Utilitzat en esborranys anteriors de l'RFC, reservat. RFC 2616
            ( CODI_307, "Redirecció temporal" ) // RFC 2616
            ( CODI_400, "Petició incorrecta" )
            ( CODI_401, "No autoritzat" )
            ( CODI_402, "Cal pagament" ) // No utilitzat, però reservat
            ( CODI_403, "Prohibit" )
            ( CODI_404, "No trobat" )
            ( CODI_405, "Mètode no permès" )
            ( CODI_406, "No acceptable" ) // Contingut conflictiu amb "Accept"
            ( CODI_407, "Cal autenticació de proxy" )
            ( CODI_408, "Temps de petició exhaurit" )
            ( CODI_409, "Conflicte" )
            ( CODI_410, "Desaparegut" )
            ( CODI_411, "Cal indicar longitud" )
            ( CODI_412, "Precondició fallida" )
            ( CODI_413, "Entitat de petició massa gran" )
            ( CODI_414, "L'URI sol·licitada és massa llarga" )
            ( CODI_415, "Media type no suportat" )
            ( CODI_416, "Rang sol·licitat no satisfactible" ) // RFC 2616
            ( CODI_417, "Expectació fallada" ) // RFC 2616
            ( CODI_500, "Error intern del servidor" )
            ( CODI_501, "No implementat" )
            ( CODI_502, "Porta d'enllaç incorrecta" )
            ( CODI_503, "Servei no disponible" )
            ( CODI_504, "Temps de porta d'enllaç exhaurit" )
            ( CODI_505, "Versió d'HTTP no suportada" )
    ; // map_list_of

} // ns anònim

namespace tfc {

namespace FrasesEstatHTTP {

const string& frase(CodiEstatHTTP codi) {
    // FRASES és constant, per tant l'operador [] no es pot fer
    //        servir directament, complicant-ne l'accés
    // find() retorna un iterador al 'pair'
    const auto it = FRASES.find(codi);
    assert( FRASES.end() != it );

    return it->second;
}

} // ns FrasesEstatHTTP

} // ns tfc

// vim:set ts=4 et ai: //
