#if !defined(_COS_PETICIO_H_)
#define _COS_PETICIO_H_

/*!
 * \file
 * \author Toni Corvera
 * \brief Interficie del cos de les respostes HTTP
 *
 * $Id: CosPeticio.h 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include <string>

namespace tfc {

/*!
 * \brief Cos dels missatges de petició
 * \sa PeticioHTTP
 */
class CosPeticio {
public:
    /*!
     * \brief Constructor per defecte
     * \param mida_esperada Mida de les dades \em predita.
     *        0 per mida desconeguda
     *        Només afecta a la reserva inicial de memòria
     */
    explicit CosPeticio(size_t mida_esperada = 0);
    //! Constructor de còpia
    CosPeticio(const CosPeticio & c)
        : buffer_(c.buffer_)
    {
        // buit
    }
    //! Operador d'assignació
    CosPeticio& operator=(const CosPeticio & c) {
        buffer_ = c.buffer_;
        return *this;
    }
    //! Obté el contingut de la petició
    const std::string& contingut() const { return buffer_; }
    //! Defineix el contingut de la petició
    void contingut(const std::string & dades);
    //! Obté la mida del contingut de la petició
    size_t mida() const { return buffer_.length(); }
    //! Afegeix contingut
    void afegeix(const std::string & dades);
    /*!
     * \brief Defineix el contingut, sense còpia de dades
     * \pre S = dades ; C = contingut();
     * \post dades == C && contingut == S
     */
    void swap(std::string & dades);
private:
    std::string buffer_;
};

} // ns tfc

#endif // _COS_PETICIO_H_

// vim:set ts=4 et ai: //
