#if !defined(_DATA_HTTP_H_)
#define _DATA_HTTP_H_

/*!
 * \file
 * \author Toni Corvera
 *
 * $Id: DataHTTP.h 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "Excepcions.h"
#include "portabilitat.h"

#include <ctime>
#include <locale>
#include <sstream>
#include <string>
#include <boost/date_time.hpp>

namespace tfc {

class ErrorFormatDataIncorrecte : public ErrorTFC {
public:
    explicit ErrorFormatDataIncorrecte(const std::string & w = "Format de data/hora incorrecte")
        : ErrorTFC(w)
    {
        // buit
    }
};

/*!
 * \brief Generació d'una data en el format estàndard HTTP
 * \note El format estàndard sempre utilitza l'horari GMT.
 * \param t Data en format \c std::time_t
 * \returns Cadena amb la data
 *
 * Exemple:
 * Dilluns 16 d'abril de 2012 a les 3:30:45, GMT+2:
 * \code
 * Mon, 16 Apr 2012 01:30:45 GMT
 * \endcode
 * \invariant \code T == llegeix_data_http(data_http(T)) \endcode
 */
std::string data_http(std::time_t t);

/*!
 * \copydoc data_http(std::time_t)
 */ // Dreçera
inline std::string data_http(const boost::posix_time::ptime & pt) {
    std::tm t(boost::posix_time::to_tm(pt));
    return data_http(std::mktime(&t));
}

/*!
 * \brief Generació de la data actual en format HTTP
 * \sa data_http(std::time_t)
 */
std::string data_http();

/*!
 * \brief Interpretació d'una data HTTP
 * \note Invers de \ref data_http(time_t)
 * \param s Representació textual de la data en un dels formats acceptats en HTTP
 * \remarks RFC 2068, 3.3.1: A l'hora d'interpretar s'han de suportar els tres formats
 *          següents: \code
 *          Sun, 06 Nov 1994 08:49:37 GMT  // RFC 822
 *          Sunday, 06-Nov-94 08:49:37 GMT // RFC 850
 *          Sun Nov 6 08:49:37 1994        // ANSI C asctime()
 * \endcode (a l'hora de generar, cal utilitzar el primer
 * \returns Data/hora extreta
 * \invariant \code T == data_http(llegeix_data_http(T)) \endcode
 */
time_t llegeix_data_http(const std::string & s) throw (ErrorFormatDataIncorrecte);

} // ns tfc

#endif // _DATA_HTTP_H_

// vim:set ts=4 et ai: //
