
/*!
 * \file
 * \author Toni Corvera
 *
 * $Id: MissatgeHTTP.hxx 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "MissatgeHTTP.h"
#include "utils.h"

#include <algorithm>
#include <cassert>
#include <boost/algorithm/string.hpp>

using namespace std;
using namespace tfc;
using boost::algorithm::to_lower_copy;

namespace {

} // ns anònim

namespace tfc {

template<typename TCos>
MissatgeHTTP<TCos>::MissatgeHTTP(const MissatgeHTTP & m)
    : capsaleres_(m.capsaleres_), cos_(m.cos_)
{
    // No cal std::copy, el constructor ja copia
    //std::copy(m.capsaleres_.begin(), m.capsaleres_.end(), capsaleres_.begin());

    // POSTCONDICIÓ
    assert(std::equal(m.capsaleres_.begin(), m.capsaleres_.end(), capsaleres_.begin()));
}

template<typename TCos>
void MissatgeHTTP<TCos>::afegeix_capsalera(const string &nom, const string &valor) {
    capsaleres_.push_back(CapsaleraHTTP(to_lower_copy(nom),valor));
}

template<typename TCos>
string MissatgeHTTP<TCos>::capsalera(const string & nom, bool * definida) const {
    if (0 != definida) {
        *definida = false;
    }
    string r;
    const string lnom = to_lower_copy(nom);
    for (auto it = capsaleres_.begin(); it != capsaleres_.end(); ++it) {
        if (it->nom() == lnom) {
            r = it->valor();
            if (0 != definida) {
                *definida = true;
            }
            break;
        }
    }
    return r;
}

template<typename TCos>
template<typename Iterador>
void MissatgeHTTP<TCos>::afegeix_capsaleres(const Iterador & inici, const Iterador & final) {
    Es_Pair_De_Strings<typename Iterador::value_type>();
    for (Iterador it = inici; it != final; ++it) {
        afegeix_capsalera(it->first, it->second);
    }
}

template<typename TCos>
struct MissatgeHTTP<TCos>::Impressor {
    std::ostream & s;
    const std::string & le; //!< Final de línia
    Impressor(std::ostream & s_, const std::string & le_) : s(s_), le(le_) {}
    void operator() (const CapsaleraHTTP & c) { s << c << le ;}
};

template<typename TCos>
void MissatgeHTTP<TCos>::imprimeix_capsaleres(ostream & os, const string & le) const {
    std::for_each(capsaleres().begin(), capsaleres().end(), Impressor(os, le));
}

template<typename TCos>
bool MissatgeHTTP<TCos>::elimina_capsalera(const string & nom) throw () {
    typedef tipus_llista_capsaleres::iterator iterator;
    const string lo_nom = to_lower_copy(nom);
    for (iterator it = capsaleres_.begin(); it != capsaleres_.end(); ++it) {
        if (to_lower_copy(it->nom()) == lo_nom) {
            capsaleres_.erase(it);
            return true;
        }
    }
    return false;
}

} // ns tfc

// vim:set ts=4 et ai: //
