
/*!
 * \file
 * \author Toni Corvera
 *
 * $Id: PeticioHTTP.cc 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "PeticioHTTP.h"

#include <boost/assign.hpp> // boost::assign::map_list_of

using namespace boost;
using namespace std;
using namespace tfc;

namespace {

const map<TipusPeticioHTTP, string> NOMS_TIPUS = assign::map_list_of
    ( GET, "GET" )
    ( POST, "POST" )
    ( HEAD, "HEAD" )
    ( OPTIONS, "OPTIONS" )
    ( TRACE, "TRACE" )
    ( PUT, "PUT" )
    ( METODE_DELETE, "DELETE" )
    ( CONNECT, "CONNECT" )
    ( desconegut, "desconegut" );
} // ns <anònim>

namespace tfc {

namespace NomTipusPeticioHTTP {

// NOMS_TIPUS és constant, per tant l'operador [] no es pot fer
// servir directament, complicant-ne l'accés
const string& nom(TipusPeticioHTTP tipus) {
    return NOMS_TIPUS.find(tipus)->second;
}

TipusPeticioHTTP tipus(const string & nom) throw (ErrorMetodeDesconegut) {
    TipusPeticioHTTP tipus = desconegut;
    // RFC 2616, 5.1.1: "The method is case-sensitive."
    if ("GET" == nom) {
        tipus = GET;
    }
    else if ("POST" == nom) {
        tipus = POST;
    }
    else if ("HEAD" == nom) {
        tipus = HEAD;
    }
    else if ("OPTIONS" == nom) {
        tipus = OPTIONS;
    }
    else if ("TRACE" == nom) {
        tipus = TRACE;
    }
    else if ("PUT" == nom) {
        tipus = PUT;
    }
    else if ("DELETE" == nom) {
        tipus = METODE_DELETE;
    }
    else if ("CONNECT" == nom) {
        tipus = CONNECT;
    }
    else {
        throw ErrorMetodeDesconegut("El mètode `"+nom+"` és desconegut per aquest servidor");
    }
    return tipus;
}

} // ns NomTipusPeticioHTTP

// Impressió de VersioHTTP
ostream& operator<<(ostream & os, VersioHTTP v) {
    string s;
    s.reserve(8);
    switch (v) {
    case HTTP_DESCONEGUT: s = "HTTP/?.?"; break;
    case HTTP_0_9: s = "HTTP/0.9"; break;
    case HTTP_1_0: s = "HTTP/1.0"; break;
    case HTTP_1_1: s = "HTTP/1.1"; break;
    case HTTP_1_X: s = "HTTP/1.x"; break;
    case HTTP_SUPERIOR: s = "HTTP/x.y"; break;
    default: // Si passa probablement v no està inicialitzat
        assert( !"Valor de VersioHTTP fora de rang" );
        s = "HTTP/?.?";
        break;
    }

    return os << s;
}

} // ns tfc

// vim:set ts=4 et ai: //
