#if !defined(_RESPOSTA_HTTP_H_)
#define _RESPOSTA_HTTP_H_

/*!
 * \file
 * \author Toni Corvera
 *
 * $Id: RespostaHTTP.h 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "CapsaleraHTTP.h"
#include "CodisEstatHTTP.h"
#include "CosResposta.h"
#include "Excepcions.h"
#include "MissatgeHTTP.h"
#include "portabilitat.h"

#include <cassert>
#include <list>
#include <map>
#include <memory>
#include <ostream>
#include <string>
#include <boost/static_assert.hpp>

namespace tfc {

/*!
 * \brief Representació d'un resposta HTTP
 */	
class RespostaHTTP : public MissatgeHTTP<CosResposta> {
public:
    /*!
     * \param codi Codi d'estat HTTP
     */
    explicit RespostaHTTP(CodiEstatHTTP codi)
        : MissatgeHTTP<CosResposta>(), codi_(codi), cos_()
	{
		// buit
	}

	//! Constructor de còpia
	RespostaHTTP(const RespostaHTTP & r)
        : MissatgeHTTP<CosResposta>(r), codi_(r.codi_), cos_(r.cos_)
	{
		// buit
	}

    bool operator==(const RespostaHTTP & r) const {
        if (this == &r) {
            return true;
        }
        const MissatgeHTTP<CosResposta> * pt = this, * pr = &r;

        return codi_ == r.codi_ &&
               cos_ == r.cos_ &&
               (*pt) == (*pr);
    }
	
	//! Codi d'estat HTTP.  \sa set_codi()
    CodiEstatHTTP codi() const { return codi_; }

	//! Frase d'estat HTTP. \todo XXX: Cal?
    std::string frase() const throw (ErrorMetodeDesconegut) {
        return FrasesEstatHTTP::frase(codi_);
	}

private:
    //! Codi d'estat HTTP.  \sa codi()
    const CodiEstatHTTP codi_;
    std::shared_ptr<CosResposta> cos_;

    /*!
     * \brief Representació textual
     * \todo XXX: No necessari? Al menys útil durant el desenvolupament
     */
    friend std::ostream& operator<< (std::ostream & os, const RespostaHTTP & r) {
        os << "RespostaHTTP {\n" << r.codi() << " " << r.frase() << "\n";
        //os << static_cast<MissatgeHTTP>(r);
        r.imprimeix_capsaleres(os);
        return os << "}";
    }
};

} // ns tfc

#endif // _RESPOSTA_HTTP_H_

// vim:set ts=4 et ai: //
