
/*!
 * \file
 * \author Toni Corvera
 *
 * $Id: TipusMIME.cc 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "TipusMIME.h"

#include <boost/unordered_map.hpp> // també a std::tr1
#include <boost/algorithm/string.hpp>
#include <boost/assign.hpp>

using namespace std;
using boost::filesystem::path;

namespace {

/*!
 * \brief Índexs d'array \c NOMS_TIPUS_CONEGUTS
 * \sa NOMS_TIPUS_CONEGUTS
 * \sa tipus_registrats
 */
enum IdxTipusConeguts {
    TEXT_PLAIN,
    TEXT_HTML,
    TEXT_CSS,
    IMAGE_JPEG,
    IMAGE_PNG,
    IMAGE_GIF,
    APPLICATION_X_OBJECT
    // , TEXT_X_M4
};

/*!
 * \brief Tipus MIME coneguts
 * \remarks Índexs a \c IdxTipusConeguts
 * \sa IdxTipusConeguts
 */
const char * NOMS_TIPUS_CONEGUTS[] = {
        "text/plain",
        "text/html",
        "text/css",
        "image/jpeg",
        "image/png",
        "image/gif",
        "application/x-object"
        // , "text/x-m4"
};


boost::unordered_map<string, string> tipus_registrats = boost::assign::map_list_of
        ( "txt", NOMS_TIPUS_CONEGUTS[TEXT_PLAIN] )
        ( "html", NOMS_TIPUS_CONEGUTS[TEXT_HTML] )
        ( "htm", NOMS_TIPUS_CONEGUTS[TEXT_HTML] )
        ( "css", NOMS_TIPUS_CONEGUTS[TEXT_CSS] )
        ( "jpg", NOMS_TIPUS_CONEGUTS[IMAGE_JPEG] )
        ( "jpeg", NOMS_TIPUS_CONEGUTS[IMAGE_JPEG] )
        ( "jpe", NOMS_TIPUS_CONEGUTS[IMAGE_JPEG] )
        ( "png", NOMS_TIPUS_CONEGUTS[IMAGE_PNG] )
        ( "gif", NOMS_TIPUS_CONEGUTS[IMAGE_GIF] )
        // Tipus per codi: text/x-c, text/x-c++, application/x-object, etc.
        // Vistos amb 'file -i' i Apache, la x indica que no són estàndards, i les dues aplicacions
        // no es posen d'acord (p.e. .h va donar x-c i x-chdr, .cc va donar x-c++ i x-c++src)
        /*
        ( "o", NOMS_TIPUS_CONEGUTS[APPLICATION_X_OBJECT] )
        ( "h", NOMS_TIPUS_CONEGUTS[TEXT_PLAIN] )
        ( "cc", NOMS_TIPUS_CONEGUTS[TEXT_PLAIN] )
        ( "ac", NOMS_TIPUS_CONEGUTS[TEXT_PLAIN] )
        */
;


const string tipus_per_defecte( NOMS_TIPUS_CONEGUTS[TEXT_PLAIN] );

} // ns anònim

namespace tfc {

namespace RegistreTipusMIME {

const string& per_extensio(const string & ext) {
    assert( ext.empty() || '.' != ext[0] ); // PRECONDICIÓ
    auto it = tipus_registrats.find(boost::algorithm::to_lower_copy(ext));
    if (it == tipus_registrats.end()) {
        return tipus_per_defecte;
    }
    return it->second;
}

void registra_extensio(const string & ext, const string & tipus) {
    const std::string clau = boost::algorithm::to_lower_copy(ext);
    tipus_registrats[clau] = tipus;
}

const string& tipus(const path & ruta) {
    string ext = ruta.extension().string();
    if (!ext.empty()) {
        ext = ext.erase(0,1);
    }
    return  RegistreTipusMIME::per_extensio(ext);
}

} // ns RegistreTipusMIME

} // ns tfc

// vim:set ts=4 et ai: //
