#if !defined(_TIPUS_MIME_H_)
#define _TIPUS_MIME_H_

/*!
 * \file
 * \brief Registre de tipus MIME
 * \author Toni Corvera
 *
 * $Id: TipusMIME.h 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "portabilitat.h"

#include <string>
#include <boost/filesystem.hpp>

namespace tfc {

/*!
 * \brief Encapsulació del registre i consulta de tipus MIME
 * \todo TODO: (opcional) proporcionar opcionalment suport de libmagic
 */
namespace RegistreTipusMIME {

/*!
 * \brief Obté el tipus MIME corresponent a una extensió.
 * \param ext Extensió (sense punt), no distingeix majúscules/minúscules
 * \pre \code '.' != *ext.begin() \endcode
 * \remarks Per tipus no registrats retorna \c text/plain
 */
// Apache i webfsd fan servir text/plain per defecte en comptes d'application/octect-stream
const std::string& per_extensio(const std::string& ext);

/*!
 * \brief Registra un tipus MIME associat a una extensió
 * \param ext Extensió (sense punt), no distingeix majúscules/minúscules
 * \param tipus Tipus MIME
 */
void registra_extensio(const std::string & ext, const std::string & tipus);

/*!
 * \brief Drecera per obtenir el tipus MIME associat a un arxiu
 */
const std::string& tipus(const boost::filesystem::path & p);

} // ns RegistreTipusMIME

} // ns tfc

#endif // _TIPUS_MIME_H_

// vim:set ts=4 et ai: //
