
/*!
 * \file
 * \author Toni Corvera
 *
 * $Id: debug.cc 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined(NDEBUG) && defined(TFC_MOSTRA_CUA_CLIENTS) // Només en mode de desenvolupament

#include "debug.h"

#include <iostream>
#include <boost/date_time.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/thread.hpp>

using namespace tfc;
using namespace std;

namespace {

using boost::lexical_cast;

//! ThreadPool per mostrar l'estat del servidor per pantalla
ThreadPool uipool(1);

//! Functor per mostrar periòdicament quants clients estan a la cua
struct LlistadorTasques {
    const ThreadPool & pool;
    explicit LlistadorTasques(const ThreadPool & tp)
        : pool(tp)
    {}
    void operator()() {
        // Executa
        const string s = "Clients en cua (sense atendre): " +
                         lexical_cast<string>(pool.num_tasques()) + "\n";
        cerr << s << flush;
        // Programa següent
        boost::this_thread::sleep(boost::posix_time::seconds(3));
        if (uipool.es_oberta()) {
            uipool.programa(LlistadorTasques(pool));
        }
    }
};

} // ns anònim

namespace tfc {

namespace debug {

void init(const ThreadPool & pool_clients) {
    uipool.programa(LlistadorTasques(pool_clients));
}

} // ns debug

} // ns tfc

#endif // NDEBUG

// vim:set ts=4 et ai: //
