
/*!
 * \file
 * \author Toni Corvera
 *
 * $Id: utils.cc 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include "Excepcions.h"
#include "utils.h"

#include <fstream>
// FIXME: No utilitzats?
#include <boost/date_time/c_local_time_adjustor.hpp>
#include <boost/date_time/local_time_adjustor.hpp>

using namespace std;
namespace fs = boost::filesystem;

std::ostream& operator<<(std::ostream & os, tfc::utils::manipulador_ostream_t fn) {
    fn(os);
    return os;
}

namespace tfc {

namespace utils {

bool es_llegible(const fs::path & p) throw () {
    // assert( fs::exists(p) ); // PRECONDICIÓ
    if (!fs::exists(p)) { // relaxada
        return false;
    }
    if (fs::is_directory(p)) {
        boost::system::error_code ec;
        fs::directory_iterator(p, ec);
        return !ec;
    }
    else if(fs::is_regular_file(p)) {
        ifstream ifs(p.string().c_str(), ifstream::in);
        return ifs.good();
    }
    else {
        // relaxada: assert( !"precondició no respectada, p no és un fitxer o directori" );
        return false;
    }
    return false;
}

bool es_canonic(const fs::path & p) throw () {
    boost::system::error_code ec;
    const bool eq = p == fs::canonical(p, ec);
    return (!ec) && eq;
}


string lpad(const string & s, size_t mida, char f) {
    const size_t L = s.length();
    const size_t c = (L < mida) ? (mida-L) : 0;
    return string(c, f) + s;
}

string rpad(const string & s, size_t mida, char f) {
    const size_t L = s.length();
    const size_t c = (L < mida) ? (mida-L) : 0;
    return s + string(c, f);
}

string formata_data(const boost::posix_time::ptime & pt, const string & fmt) {
    using namespace boost::gregorian;
    using namespace boost::posix_time;
    ostringstream ss;
    time_facet * df = new time_facet(); // No destruir, la locale se n'apropia
    ss.imbue(std::locale(std::locale::classic(), df)); // classic() => locale "C"
    df->format(fmt.c_str());
    ss << pt;
    return ss.str();
}

string formata_data(const std::time_t pt, const string & fmt) {
    return formata_data(boost::posix_time::from_time_t(pt), fmt);
}

boost::posix_time::ptime llegeix_data(const string & s, const string & fmt) {
    throw ErrorNoImplementat(FITXER_I_LINIA_); // FIXME
    using namespace boost::posix_time;
    istringstream iss(s);
    time_input_facet * tif = new time_input_facet(fmt.c_str());
    iss.imbue(std::locale(std::locale::classic(), tif));
    boost::posix_time::ptime pt;
    iss >> pt;
    return pt;
}

} // ns utils

} // ns tfc

// vim:set ts=4 et ai: //
