
/*!
 * \file
 * \brief Tests de les restriccions definides a \ref tfc::utils
 * \remarks Les restriccions són evaluades en el moment de la compilació (i eliminades
 *          per l'optimització), per tant el fet que aquest mòdul compili és l'indicatiu
 *          que s'han superat els tests
 *
 * \author Toni Corvera
 *
 * $Id: TestConstraints.cc 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include <functional>
#include <iostream>
#include <istream>
#include <map>
#include <ostream>

#include <utils.h>

#include "comu.h"

using namespace std;
using namespace tfc;

template<typename T>
struct CompatibleAmbPair {
    const T first, second;
    CompatibleAmbPair(T&t1, T&t2) : first(t1), second(t2) {}
};

struct Functor {
    void operator()() {}
};
struct Classe {
    void metode(){}
};

void fnvoid() {}

struct AmbOperadorsInExExterns {
};
// Operadors externs
ostream& operator<<(ostream& s, const AmbOperadorsInExExterns &) {
    return s;
}
istream& operator>>(istream & s, const AmbOperadorsInExExterns &) {
    return s;
}

BOOST_AUTO_TEST_SUITE( Constraints_TestSuite )

BOOST_AUTO_TEST_CASE( constraints )
{
    Es_Pair_De_Strings< pair<string, string> >();
    Es_Pair_De_Strings< CompatibleAmbPair<string> > ();

    Es_Pot_Cridar_Sense_Arguments<Functor>();
    Es_Pot_Cridar_Sense_Arguments<void(*)()>();
    Es_Pot_Cridar_Sense_Arguments<void (void)>();
#if __cplusplus >= 201103L
    // Codi C++11
    Es_Pot_Cridar_Sense_Arguments<decltype(fnvoid)>();
#endif

    Te_Operador_Insercio<AmbOperadorsInExExterns>();
    Te_Operador_Extraccio<AmbOperadorsInExExterns>();
}

BOOST_AUTO_TEST_SUITE_END()

// vim:set ts=4 et ai: //
