
/*!
 * \file
 * \brief Tests de tfc::DataHTTP.
 * \sa tfc::DataHTTP
 *
 * \author Toni Corvera
 *
 * $Id: TestDataHTTP.cc 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <boost/date_time.hpp>

#include <DataHTTP.h>
#include <utils.h>

#include "comu.h"

using namespace std;
using namespace tfc;

namespace tfc { namespace Fixtures {

struct FxtData {
    // Data de referència
    const time_t epoch; //!< Epoch UNIX
    const boost::posix_time::ptime pt_epoch; //!< Epoch UNIX
    const string str_epoch; //!< Representació correcta de l'Epoch UNIX

    FxtData()
        : epoch(0),
            // FIXME: 01 ó 1?
          str_epoch("Thu, 01 Jan 1970 00:00:00 GMT"),
          pt_epoch(boost::posix_time::from_time_t(0))
    {
        // buit
    }
};

} // ns Fixtures
} // ns tfc

BOOST_FIXTURE_TEST_SUITE( DataHTTP_TestSuite, Fixtures::FxtData )

BOOST_AUTO_TEST_CASE( representacio_i_analisi )
{
    using namespace boost::posix_time;
    // Valors de referència (Epoch)
    BOOST_CHECK_EQUAL( data_http(epoch), str_epoch );
    BOOST_CHECK_EQUAL( llegeix_data_http(str_epoch), epoch );
    // Invariants
    const string ara = data_http();
    const time_t t_ara = llegeix_data_http(ara);
    BOOST_CHECK_EQUAL( data_http(llegeix_data_http(ara)), ara );
    BOOST_CHECK_EQUAL( llegeix_data_http(data_http(t_ara)), t_ara );
}

BOOST_AUTO_TEST_CASE( formats_reconeguts )
{
    // Valors d'exemple de l'RFC 2068, 3.3.1:
    const char * rfc822 = "Sun, 06 Nov 1994 08:49:37 GMT"; // RFC 822, updated by RFC 1123
    const char * rfc850 = "Sunday, 06-Nov-94 08:49:37 GMT"; // RFC 850, obsoleted by RFC 1036
    const char * casctime = "Sun Nov  6 08:49:37 1994"; // ANSI C's asctime() format
    // Format tècnicament incorrecte, comprobació que s'accepten els
    // dos dígits en el día correctament
    const char * casctime_alt = "Sun Nov 06 08:49:37 1994";
    BOOST_CHECK_NO_THROW( llegeix_data_http(rfc822) );
    BOOST_CHECK_NO_THROW( llegeix_data_http(rfc850) );
    BOOST_CHECK_NO_THROW( llegeix_data_http(casctime) );

    BOOST_CHECK_EQUAL( llegeix_data_http(rfc822), llegeix_data_http(rfc850) );
    BOOST_CHECK_EQUAL( llegeix_data_http(rfc822), llegeix_data_http(casctime) );
    BOOST_CHECK_EQUAL( llegeix_data_http(casctime), llegeix_data_http(casctime_alt) );
}

BOOST_AUTO_TEST_SUITE_END()

// vim:set ts=4 et ai: //
