
/*!
 * \file
 * \brief Tests de tfc::PeticioHTTP.
 * \sa tfc::PeticioHTTP
 *
 * \author Toni Corvera
 *
 * $Id: TestPeticioHTTP.cc 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <sstream>
#include <string>

#include <CapsaleraHTTP.h>
#include <Excepcions.h>
#include <PeticioHTTP.h>
#include <Uri.h>

#include "comu.h"

using namespace std;
using namespace tfc;

namespace tfc { namespace Fixtures {

struct FxtPeticio {
    FxtPeticio()
        : get1(GET, Uri("/"), HTTP_1_1),
          get2(GET, Uri("/"), HTTP_1_1),
          post(POST,Uri("/"), HTTP_1_0),
          http_1_1("HTTP/1.1")
    {
        BOOST_TEST_MESSAGE("Fixture setup");
    }
    ~FxtPeticio()
    {
        BOOST_TEST_MESSAGE("Fixture teardown");
    }
    PeticioHTTP get1, get2, post;
    string http_1_1;
};

} // ns Fixtures
} // ns tfc

//BOOST_AUTO_TEST_SUITE( PeticioHTTP_TestSuite )
BOOST_FIXTURE_TEST_SUITE( PeticioHTTP_TestSuite, Fixtures::FxtPeticio )

BOOST_AUTO_TEST_CASE( igualtat )
//BOOST_FIXTURE_TEST_CASE( igualtat, PeticioFixture )
{
    BOOST_CHECK( get1 == get2 );
}

BOOST_AUTO_TEST_CASE( copia )
{
    PeticioHTTP c(post);
    BOOST_CHECK_EQUAL( c, post );
}

BOOST_AUTO_TEST_CASE( getters )
{
    BOOST_CHECK_EQUAL(get1.tipus(), GET);
    BOOST_CHECK_EQUAL(get1.versio(), HTTP_1_1);
    BOOST_CHECK_EQUAL(get1.ruta(), Uri("/"));
}

BOOST_AUTO_TEST_CASE( MostraImpressio )
{
    cerr << "Mostra d'impressió de peticions HTTP:" << endl;
    PeticioHTTP get3(get1);
    get3.afegeix_capsalera("Host", "www.example.com");
    cerr << get3 << endl;
}

BOOST_AUTO_TEST_CASE( noms )
{
    BOOST_CHECK_EQUAL( NomTipusPeticioHTTP::nom(GET), "GET" );
    BOOST_CHECK_EQUAL( NomTipusPeticioHTTP::nom(POST), "POST" );
    BOOST_CHECK_EQUAL( NomTipusPeticioHTTP::nom(HEAD), "HEAD" );
    BOOST_CHECK_EQUAL( NomTipusPeticioHTTP::nom(OPTIONS), "OPTIONS" );
    BOOST_CHECK_EQUAL( NomTipusPeticioHTTP::nom(TRACE), "TRACE" );
    BOOST_CHECK_EQUAL( NomTipusPeticioHTTP::nom(PUT), "PUT" );
    BOOST_CHECK_EQUAL( NomTipusPeticioHTTP::nom(METODE_DELETE), "DELETE" );
    BOOST_CHECK_EQUAL( NomTipusPeticioHTTP::nom(CONNECT), "CONNECT" );
}

#if 0
BOOST_AUTO_TEST_CASE( setters )
{
}
#endif

BOOST_AUTO_TEST_SUITE_END()

// vim:set ts=4 et ai: //
