
/*!
 * \file
 * \brief Proves de transformacions
 *
 * \author Toni Corvera
 *
 * $Id: transformacions.cc 483 2013-06-17 17:41:04Z toni $
 */
/*
 * Copyright (c) 2012 Toni Corvera
 *
 * This file is part of TFCWeb.
 *
 * TFCWeb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TFCWeb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TFCWeb. If not, see <http://www.gnu.org/licenses/>.
 */

#include <Transformacions.h>

#include "comu.h"

#include <iostream>
#include <fstream>
#include <memory>

using namespace tfc;
using namespace std;

#define FILE_DEFLATED __FILE__ ".deflate"

int main(int, char **, char **) {

    // Fitxer de referència. Servidor.cc és el fitxer de codi més gran
    ifstream ifs("../src/Servidor.cc"); //__FILE__);
    ofstream ofs_deflated(FILE_DEFLATED);
    if (ifs.fail() || ofs_deflated.fail()) {
        assert( !"Error inesperat" );
    }
    stringstream ss;

    // Extracció i escriptura de mostra
    FuncioExtractoraIstream fx_ifs(ifs);
    FuncioExtractoraIstream fx_ss(ss);
    FuncioEscriptoraOstream fs_cout(cout);
    FuncioEscriptoraOstream fs_ss(ss);
    FuncioEscriptoraOstream fs_ofs_deflated(ofs_deflated);

    TransformacioChunked chunked;
    TransformacioDeflate deflate;

    AplicadorTransformacio<TransformacioChunked> chunked_ifs_a_cout(fx_ifs, fs_cout);
    AplicadorTransformacio<TransformacioChunked> chunked_ifs_a_ss(fx_ifs, fs_ss);

    AplicadorTransformacio<TransformacioDeflate> deflate_ifs_a_ofs(fx_ifs, fs_ofs_deflated);
    AplicadorTransformacio<TransformacioDeflate> deflate_ifs_a_ss(fx_ifs, fs_ss);

    EstatTransformacioInversa ok;
    cout << boolalpha;

    // Transformació chunked
    {
        cout << "----------------- CHUNKED -----------------" << endl;
        chunked_ifs_a_cout.aplica();
        ifs.seekg(0, ios::beg);
        cout << ">> FINAL DE CHUNKED <<" << endl;
    }

    // Transformació chunked inversa (des-transformació)
    {
        chunked_ifs_a_ss.aplica(); // Còpia a ss
        const string txt_chunked = ss.str();
        ifs.seekg(0, ios::beg);
        ss.str("");
        cout << "----------------- CHUNKED INVERS -----------------" << endl;
        cout << chunked.destransforma(txt_chunked, ok);
        cout << ">> FINAL DE CHUNKED INVERS. OK: " << ok << " <<" << endl;
    }

    // Transformació deflate
    {
        cout << "----------------- DEFLATE (" FILE_DEFLATED ") -----------------" << endl;
        deflate_ifs_a_ofs.aplica();
        ifs.seekg(0, ios::beg);
        cout << ">> FINAL DE DEFLATE <<" << endl;
    }

    // Transformació deflate inversa
    {
        deflate_ifs_a_ss.aplica(); // Còpia a ss
        const string txt_deflated = ss.str();
        ifs.seekg(0, ios::beg);
        ss.str("");
        cout << "----------------- DEFLATE INVERS (" FILE_DEFLATED ") -----------------" << endl;
        cout << deflate.destransforma(txt_deflated, ok);
        cout << ">> FINAL DE DEFLATE INVERS. OK: " << ok << " <<" << endl;
    }

    // Transformació deflate amb fitxer no textual
    {
        // TODO: re-implementar
#if 0 // Codi anterior a neteja:
    // Sobre fitxer amb dades binàries
    { // COMPRESSIÓ
        ifstream ifsbin("URANDOM", ios::in|ios::binary);
        ofstream ofsbin("URANDOM.deflated", ios::out|ios::binary);
        FuncioExtractoraIstream in(ifsbin);
        FuncioEscriptoraOstream out(ofsbin);
        TransformacioDeflate tdfbin(in, out);
        tdfbin.aplica();
    }
    { // DESCOMPRESSIÓ
        ifstream ifsbin("URANDOM", ios::in|ios::binary);
        stringstream tmpss;
        FuncioExtractoraIstream in(ifsbin);
        FuncioEscriptoraOstream out(tmpss);
        TransformacioDeflate tdfbin(in, out);
        tdfbin.aplica();
        // tmpss conté una còpia comprimida de URANDOM
        ofstream ofsbin_inf("URANDOM.inflated", ios::out|ios::binary);
        //FuncioExtractoraIstream in(tmpss);
        //FuncioEscriptoraOstream out_inf(ofsbin_inf);
        const string buf = tdfbin.destransforma(tmpss.str(), ok);
        ofsbin_inf.write(buf.c_str(), buf.length());
        cout << "OK: " << ok << endl;
    }
#endif


    }

    return 0;
}

// vim:set ts=4 et ai: //
